// Copyright 1999 (c) by Perforce Software, Inc.  All rights reserved.
//
// p4wFileBrowserPane:
//   The file browser pane.

#ifndef p4wFileBrowserPane_H
#define p4wFileBrowserPane_H


//
// Includes
#include "p4wPane.h"


//
// External classes
class StrDict;


//
// p4wFileBrowserPane class
class p4wFileBrowserPane : public p4wPane {

public:
	//
	// Constructors and destructor.
	p4wFileBrowserPane( p4wView & ParentView, p4wRequest & Request, StrBuf & clientRoot );
	virtual			~p4wFileBrowserPane();

	//
	// Render functions.
	virtual	void		Begin();
	virtual	void		Render( StrDict * varList );
	virtual	void		End();
	virtual void		RenderError( char *e, int escapeHTML );
	
	//
	// Is this file writeable?
	int			IsWriteable() { return fIsWriteable; };

	//
	// Is this file available for editing?
	int			IsEditable() { return fIsEditable; };
	
	//
	// Others may find the head revision useful
	const char		*GetHeadRev() { return fHeadRev.Text(); };
	
	//
	// Others may find the head chg# useful
	const char		*GetHeadChange() { return fHeadChange.Text(); };
	
	//
	// Others may find the Workspace Filename useful
	const char		*GetWorkspaceFile() { return fWorkspaceFile.Text(); };
	
	//
	// Is this file in the depot?
	int			InDepot() { return fInDepot; }
	
	//
	// Does this file have a head type?
	int			HasHeadType() { return fHasHeadType; }
	
	void			HandleError( Error *e );

private:
	int 			RenderFileLink( StrPtr *file );

	void			doWSFileState();
	int 			doFileDirLink( const StrPtr *sr, 
					const StrPtr *rev, int showhide=0 );
	void			startPage( StrDict * varList=0, int mushow=0 );
	void			startTables( int showhide=0 );
	void			endTables();
	int 			NotOpenedByOwner( const StrPtr *owner );
	
	int			fIsWriteable;
	int			fIsEditable;
	int			fBeenHere;
	int			fSeenBegin;
	int			fInDepot;
	int			fBoxStarted;
	int			fHasHeadType;
	int			fShowHide;
	StrBuf		fHeadRev;
	StrBuf		fHeadChange;
	StrBuf		fWorkspaceFile;
	StrBuf		fClientRoot;
};

#endif // p4wFileBrowserPane_H
