//
// Copyright 2001 Perforce Software.  All rights reserved.
//
// This file is part of Perforce - the FAST SCM System.
//
// p4wLogoutPane:
//   Generates a result pane containing the result of a simple
//   command or a spec -i command.


#ifndef p4wLogoutPane_H
#define p4wLogoutPane_H

#include "p4wPane.h"

class p4wLogoutPane : public p4wPane {

public:

	p4wLogoutPane( p4wView & ParentView, 
		p4wRequest & Request, const char *title );
		
	//
	// special constructor used to replace a field
	// in a spec -i (InputData) command
	
	p4wLogoutPane( p4wView & ParentView, 
		p4wRequest & Request, const char *title, const char *field,
		const char *val );
				
	virtual		~p4wLogoutPane();

	virtual	void	Begin();
	virtual	void	RenderInfo( char *data, char level );
	virtual	void	RenderError( char *data, int escapeHTML );
	virtual	void	End();
	
	virtual void	InputData( StrBuf * strbuf, Error * e );
	
	//
	// Override HandleError() so that we can generate
	// a better error message for labelsync's empty string
	// error
	
	virtual void	HandleError( Error *e );
	
	//
	// LogoutPane-specification functions.
	
	void		AdditionalFunction();
	void		SkipEnd() { fSkipEnd = 1; };

protected:

	int		fSeenBegin;
	int		fFunctionCount;
	
private:

	//
	// Call OutputDirectoryHeader to generate linkified path
	// for certain commands
	
	void		doDirHeader();
	
	//
	// Construct a title for this status page
	
	void		doTitle( StrBuf &title );
	
	//
	// Insure we only do End() processing once at the end
	
	int		fSkipEnd;
	StrBuf		fTitle;
	
	// 
	// Special handling for replacing a field of a form
	
	int		fReplaceField;
	StrBuf		fField;
	StrBuf		fVal;
};

#endif // p4wLogoutPane_H
