//
// Copyright 2007 Perforce Software.  All rights reserved.
//
// This file is part of Perforce - the FAST SCM System.
//
// p4wResolveIAProcessorPane:
//   Runs the real p4 resolve command

#include <p4wp4.h>

#include "p4wStrBuf.h"
#include "p4wHtml.h"

#include "p4wPane.h"
#include "p4wResolveIAProcessorPane.h"


p4wResolveIAProcessorPane::p4wResolveIAProcessorPane( p4wView & ParentView, 
			p4wRequest & Request, char **pErrMsg, StrBuf *pmd5in, p4wStrBuf *peditbuf, int rc )
	: p4wPane( ParentView, Request ), fSeenData(0)
{
	fErrMsg = pErrMsg;
	md5in = pmd5in;
	editbuf = peditbuf;
	fRC = rc;
}

p4wResolveIAProcessorPane::~p4wResolveIAProcessorPane()
{
}



// -------------------------------------
// Render functions.
//
void p4wResolveIAProcessorPane::Begin()
{
}

int p4wResolveIAProcessorPane::Resolve( ClientMerge *m, Error *e )
{
	fSeenData = 1;
	if (fRC)
		return fRC;

	FileSys *f = m->GetResultFile();

	if (!fRequest.isLocalRequest() && (f->Stat() & FSF_SYMLINK) && !SEC_ALLOW_CR8CHG_SYMLINKS)
	{
		*fErrMsg = "A Symlink may not be written by a remote browser.";
		return fRC = CMS_SKIP;
	}

	*fErrMsg = WriteMergedFile(f, e);

	if (*fErrMsg)
		return fRC = CMS_SKIP;

	StrBuf md5;
	f->Digest(&md5, e);
	if (!e->Test() && md5 == *md5in)
		return fRC = CMS_MERGED;
	e->Clear();
	return fRC = CMS_EDIT;
}


char * p4wResolveIAProcessorPane::WriteMergedFile(FileSys *f, Error *e)
{
	StrBuf buf;
	StrBuf md5;

	f->Digest(&md5, e);

	if(e->Test() || md5 != *md5in)
		return "Bad security key";

	f->WriteFile( editbuf, e );

	if (e->Test())
		return "Unable to write new merge file.";

	return 0;
}

void p4wResolveIAProcessorPane::End()
{
}

void p4wResolveIAProcessorPane::RenderInfo( char *data, char level )
{
}

void p4wResolveIAProcessorPane::RenderError( char *data, int escapeHTML )
{
	fErrBuf << data;
	*fErrMsg = fErrBuf.Text();
}
